//
//  PDFViewer.m
//  PDFViewer
//
//  Created by Markus Bröker on 27.03.17.
//  Copyright © 2017 Markus Bröker. All rights reserved.
//

#import "PDFViewer.h"

@implementation PDFViewer

/* Eintrittspunkt eines (N)ext (I)nterface! */
- (void)awakeFromNib {
    synthesizer = [[NSSpeechSynthesizer alloc] initWithVoice:[NSSpeechSynthesizer defaultVoice]];
}

/* Lese den Text aus dem aktuell geöffneten Dokument aus */
- (NSMutableString *)getTexts {
    NSMutableString *strings = [[NSMutableString alloc] init];
    NSUInteger nOps = [[self document] pageCount];
    
    [strings appendString:[NSString stringWithFormat:@"Genosse Parteisekretär, das Dokument hat %lu Seiten", nOps]];
    
    for (int pages = 0; pages < nOps; pages++) {
        PDFPage *page = [[self document] pageAtIndex:pages];
        [strings appendString:[self replaceString:[page string]]];
    }
    
    return strings;
}

/* Datei-Menü: Öffnen-Dialog */
- (IBAction)openDocument:(id)sender {
    NSArray *fileTypes = [NSPDFImageRep imageTypes];
    
    NSOpenPanel* panel =[NSOpenPanel openPanel];
    [panel setAllowedFileTypes:fileTypes];
    
    [panel beginWithCompletionHandler:^(NSInteger result){
        if (result == NSFileHandlingPanelOKButton) {
            
            // Audioausgabe beenden
            [self silence];
            
            // neues PDF laden
            NSURL* url = [[panel URLs] objectAtIndex:0];
            PDFDocument *doc = [[PDFDocument alloc] initWithURL:url];
            
            [self setDocument:doc];
            [self startSpeaking:nil];
        }
    }];
}

/* Datei-Menü: Schließen-Dialog */
- (IBAction)performClose: (id)sender {
    [self silence];
    [self setDocument:nil];
}

/* Button Action startSpeaking */
- (IBAction)startSpeaking:(id)sender {
    if (!synthesizer.isSpeaking) {
        NSMutableString *string = [self getTexts];
        [self speak:string];
    } else {
        [self silence];
    }
}

/* Button Action stopSpeaking */
- (IBAction)stopSpeaking:(id)sender {
    [self silence];
}

/* Sprachausgabe an */
- (void)speak:(NSMutableString *)texts {
    // Die Audioausgabe muss immer gestoppt werden, bevor Sie erneut gestartet werden darf!
    [self silence];
    
    [synthesizer startSpeakingString:texts];
}

/* Sprachausgabe aus */
- (void)silence {
    if ([synthesizer isSpeaking]) {
        [synthesizer stopSpeaking];
    }
}

/* Standard-Ersetzungen für die Sprachausgabe */
- (NSString *)replaceString:(NSString *)inputStr {
    NSDictionary *replacements = @{
        @"SED" : @"S.E.D.",
        @"DDR" : @"D.D.R.",
        @" IM ": @" I.M. ",
    };
    
    for (id key in replacements) {
        inputStr = [inputStr stringByReplacingOccurrencesOfString:key withString:replacements[key]];
    }
    
    return inputStr;
}

@end
